/*
  Copyright 2020 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
@file:JvmName("RxApiAuthClientKt")
package com.kakao.sdk.partner.auth

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.os.ResultReceiver
import com.kakao.sdk.auth.AuthApiClient
import com.kakao.sdk.auth.RxAuthApiClient
import com.kakao.sdk.auth.RxAuthApiClient.Companion.handleAuthError
import com.kakao.sdk.auth.model.OAuthToken
import com.kakao.sdk.auth.network.rxKauth
import com.kakao.sdk.common.KakaoSdk
import com.kakao.sdk.common.model.KakaoSdkError
import com.kakao.sdk.common.network.ApiFactory
import com.kakao.sdk.common.network.RxOperations
import com.kakao.sdk.common.util.SdkLog
import com.kakao.sdk.partner.auth.model.*
import io.reactivex.*

private val authApi: RxPartnerAuthApi
    get() = ApiFactory.rxKauth.create(RxPartnerAuthApi::class.java)

/**
 * 그룹에 속해 있는 다른 앱의 refreshToken으로 사용자를 식별하여 신규 토큰을 발급.
 *
 * 인증에 사용된 refreshToken 값이 앱 간에 서로 공유되는 것이 아닙니다. 이 기능를 사용하면 내 앱을 위한 새로운 [OAuthToken] 세트가 발급됩니다.
 *
 * @param groupRefreshToken 사용자 인증을 위한 다른 앱의 refreshToken.
 * @return [OAuthToken]을 방출하는 [Single] 반환.
 */
fun RxAuthApiClient.issueAccessToken(groupRefreshToken: String): Single<OAuthToken> =
    authApi.issueAccessToken(
        groupRefreshToken,
        clientId = applicationInfo.appKey,
        androidKeyHash = contextInfo.signingKeyHash,
        approvalType = approvalType.value
    )
        .compose(handleAuthError())
        .map { OAuthToken.fromResponse(it) }
        .doOnSuccess { tokenManagerProvider.manager.setToken(it) }


/**
 * 연령 인증 요청
 *
 * @param context    CustomTabs를 실행하기 위한 현재 Activity context
 * @param ageLimit   연령제한 (만 나이 기준)
 * @param authLevel  연령인증 레벨 (1차 인증 : 실명/생년월일 인증, 2차 인증 : 휴대폰 본인 인증을 통한 통신사 명의자 인증)
 * @param skipTerms  동의 화면 출력 여부
 * @param adultsOnly 서비스에서 청소년유해매채물 인증 필요 여부
 * @param underAge   연령인증 페이지 구분
 *
 */
@JvmOverloads
fun RxAuthApiClient.verifyAge(
    context: Context,
    ageLimit : Int? = null,
    authLevel : AgeAuthLevel? = null,
    skipTerms : Boolean? = null,
    adultsOnly : Boolean? = null,
    underAge : Boolean? = false
) : Completable =
    RxAgeAuthApiClient.instance.verifyAgeWithAuthentication(context, ageLimit, authLevel, skipTerms, adultsOnly, underAge).ignoreElement()

